# Scenario 'pvss': sharing a secret in parts with publicly verifiable proofs

This scenario contains statements to cryptographically share a secret in parts, to publicly check the validity of the parts, and to recompose it. In the scenario, we assume we have an issuer and some participants. The cryptography is based on Lagrange's interpolation and on the BLS12-381 elliptic curve. It offers:

 - A configurable amount *total* of shares which the secret has to be split into (it corresponds to the number of participants);
 - A configurable *quorum*, which is the minimum amount of shares needed to recompose the secret.

## Initialization

In the first phase of the protocol, each participant needs to create a keypair.
The following script generates a private key:

[](../_media/examples/zencode_cookbook/pvss/create_pvss_sk.zen ':include :type=code gherkin')

After executing the script above, the participant creates the public key like so:

[](../_media/examples/zencode_cookbook/pvss/create_pvss_pk.zen ':include :type=code gherkin')

The output should look like:

[](../_media/examples/zencode_cookbook/pvss/create_pvss_pk.json ':include :type=code json')

## Distribution of the public shares

In the second phase of the protocol, the issuer does the following actions:

- it collects all the public keys of the participants;
- it defines the *secret*;
- it defines *total* and *quorum*;
- it creates a public share for each participant;
- it creates a proof for the validity of all the public shares.

### Issuer creates the public shares

Once the issuer collects all the public keys, it should have an input which looks like:

[](../_media/examples/zencode_cookbook/pvss/pvss_pub_data.json ':include :type=code json')

In order to create the public shares and the proof, the issuer executes the following script:

[](../_media/examples/zencode_cookbook/pvss/pvss_create_shares.zen ':include :type=code gherkin')

The output should look like:

[](../_media/examples/zencode_cookbook/pvss/pvss_issuer_proof.json ':include :type=code json')

### Anyone verifies the public shares

Given the output found in the previous step, anyone can verify the public shares using the following script:

[](../_media/examples/zencode_cookbook/pvss/pvss_verify_enc_shares.zen ':include :type=code gherkin')

If no errors are returned, then the public shares have been correctly verified.

## Reconstruction of the secret

In the third and last phase of the protocol, each participant does the following actions:

- it computes its own secret share starting from its public one;
- it creates a proof of the validity of its secret share;
- it collects (some of) the secret shares provided by the other participants and it verifies them;
- if it collects at least *quorum* verified secret shares, then it reconstructs the *secret*.

### Participant creates one secret share

In this step, the participant loads all the public shares together with its own keypair, which looks like:

[](../_media/examples/zencode_cookbook/pvss/pvss_decrypt_data.json ':include :type=code json')

Then, the participant executes the following script:

[](../_media/examples/zencode_cookbook/pvss/pvss_decrypt_shares.zen ':include :type=code gherkin')

The output should look like:

[](../_media/examples/zencode_cookbook/pvss/pvss_partecipant_proof.json ':include :type=code json')

### Participant verifies secret shares of the others

Each participant now collects some secret shares provided by other participants in a json which looks like:

[](../_media/examples/zencode_cookbook/pvss/pvss_ver_decrypt_data.json ':include :type=code json')

With such input, the participant can now execute the following script:

[](../_media/examples/zencode_cookbook/pvss/pvss_ver_decrypt_shares.zen ':include :type=code gherkin')

The output should look like:

[](../_media/examples/zencode_cookbook/pvss/pvss_verified_shares.json ':include :type=code json')

### Participant reconstructs the secret

At this step, using the output of the last step and the *quorum* extracted from the public shares, the participant can execute the following script:

[](../_media/examples/zencode_cookbook/pvss/pvss_reconstruct_secret1.zen ':include :type=code gherkin')

The output should look like:

[](../_media/examples/zencode_cookbook/pvss/pvss_secret1.json ':include :type=code json')

# The script used to create the material in this page

All the scripts and the data you see in this page are generated by the pvss script [pvss.bats](https://github.com/dyne/Zenroom/blob/master/test/zencode/pvss.bats). If you want to run the script (on Linux) you should: 
 - *git clone https://github.com/dyne/Zenroom.git*
 - install **zsh** and **jq**
 - download a [zenroom binary](https://zenroom.org/#downloads) and place it */bin* or */usr/bin* or in *./Zenroom/src*
