
The Schnorr signature is a digital signature algorithm which was described by Claus Schnorr in 1989 and it was patented until February 2008. The scheme security is based on the supposed intractability of the Discrete Logarithm problem. In 2018 its Elliptic-curve version was proposed to substitue ECDSA in the bitcoin universe, this was done in the [BIP-340](https://github.com/bitcoin/bips/blob/master/bip-0340.mediawiki). In **Zenroom** you can find its implementation over BLS12-381 Elliptic-curve.

# Key generation

On this page we prioritize security over easy of use, therefore we have chosen to keep some operations separated. Particularly the generation of private and public key, which can indeed be merged into one script, but you would end up with both the keys in the same output. 

## Private key
The script below generates a **schnorr** private key. 

[](../_media/examples/zencode_cookbook/schnorr/Schnorr_createprivatekey.zen ':include :type=code gherkin')

The output should look like this: 

[](../_media/examples/zencode_cookbook/schnorr/Alice_Schnorr_privatekey.keys ':include :type=code json')

### Generate a private key from a known seed 

Key generation in Zenroom uses by default a pseudo-random as seed, that is internally generated. 

You can also opt to use a seed generated elsewhere, for example by using the [keypairoom](https://github.com/ledgerproject/keypairoom) library or it's [npm package](https://www.npmjs.com/package/keypair-lib). Suppose you end with a **Schnorr private key**, like:

[](../_media/examples/zencode_cookbook/schnorr/Schnorr_readsecretkeys.keys ':include :type=code json')

Then you can upload it with a script that look like the following script:

[](../_media/examples/zencode_cookbook/schnorr/Schnorr_upload_key.zen ':include :type=code gherkin')

Here we simply print the *keyring*.

## Public key 

Once you have created a private key, you can feed it to the following script to generate the **public key**:

[](../_media/examples/zencode_cookbook/schnorr/Schnorr_createpublickey.zen ':include :type=code gherkin')

The output should look like this: 

[](../_media/examples/zencode_cookbook/schnorr/Alice_Schnorr_pubkey.json ':include :type=code json')

# Signature

In this example we'll sign three objects: a string, a string array and a string dictionary, that we'll verify in the next script. Along with the data to be signed, we'll need the private key. The private key is in the file we have generate with the first script, while the one with the messages that we will sign is the following:

[](../_media/examples/zencode_cookbook/schnorr/message.json ':include :type=code json')

The script to **sign** these object look like this:

[](../_media/examples/zencode_cookbook/schnorr/Schnorr_sign.zen ':include :type=code gherkin')

And the output should look like this:

[](../_media/examples/zencode_cookbook/schnorr/Alice_Schnorr_sign.json ':include :type=code json')

You can now merge this file with the one where your schnorr public key is, so that the verifier has everything he needs in one file. You can do it by either use the bash command:

```bash
jq -s '.[0]*[1]' pubkey.json signature.json | tee data.json
```

where pubkey.json contains the output of the third script and signature.json the output of the above script, or by adding two rows in the previous script, one where you compute the schnorr public key and the other where you print it.

# Verification

In this section we will verify the signatures produced in the previous step. As mentioned above what we will need are the signatures, the messages and the signer public key. So the input file should look like:

[](../_media/examples/zencode_cookbook/schnorr/Alice_data.json ':include :type=code json')

The script to verify these signatures is the following:

[](../_media/examples/zencode_cookbook/schnorr/Schnorr_verifysign.zen ':include :type=code gherkin')

The result should look like:

[](../_media/examples/zencode_cookbook/schnorr/Schnorr_verifysign.json ':include :type=code json')


# The script used to create the material in this page

All the smart contracts and the data you see in this page are generated by the scripts [generic_schnorr.bats](https://github.com/dyne/Zenroom/blob/master/test/zencode/generic_schnorr.bats) . If you want to run the scripts (on Linux) you should: 
 - *git clone https://github.com/dyne/Zenroom.git*
 - install **zsh** and **jq**
 - download a [zenroom binary](https://zenroom.org/#downloads) and place it */bin* or */usr/bin* or in *./Zenroom/src*
